﻿--[[

	Quest Completist - qcCollect.lua
	Written by: Alistair Maxwell

--]]

function qcWipeWorkingDB()

	wipe(qcWorkingDB)
	wipe(qcWorkingLog)

end

function qcMergeDB()

	qcMergeSVData()
	qcSetClassBits()
	qcSetQuestCategories()

end

function qcSetQuestCategories()

	local qcRecordCount = 0
	local qcInvalidCount = 0
	local qcChangeCount = 0
	table.insert(qcWorkingLog, "-----------------------")
	table.insert(qcWorkingLog, "PROCESSING ZONE ID's...")
	table.insert(qcWorkingLog, "-----------------------")

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		qcRecordCount = (qcRecordCount + 1)
		if (qcEntry[2][1] == 99999) then
			qcInvalidCount = (qcInvalidCount + 1)
			for qcCategoryIndex, qcCategoryEntry in ipairs(qcQuestCategories) do
				if (qcCategoryEntry[2] == qcEntry[1][4]) then
					qcChangeCount = (qcChangeCount + 1)
					table.insert(qcWorkingLog, "ZONE ID UPDATED FOR QUEST: " .. tostring(qcEntry[1][2]) .. " [" .. tostring(qcEntry[1][1]) .. "]")
					table.insert(qcWorkingLog, "    ZONE ID - ORIGINAL VALUE: '" .. tostring(qcEntry[2][1]) .. "' NEW VALUE: '" .. tostring(qcCategoryEntry[1]) .. "'")
					qcEntry[2][1] = qcCategoryEntry[1]
					break
				end
			end
		end
	end

	table.insert(qcWorkingLog, "NO VALID ZONE ID FOUND FOR THE FOLLOWING CATEGORIES:")
	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		if (qcEntry[2][1] == 99999) then
			table.insert(qcWorkingLog, tostring(qcEntry[1][4]))
		end
	end

	print(tostring(qcRecordCount) .. " records were checked for invalid zone ID's. " .. tostring(qcInvalidCount) .. " invalid records were found. " .. tostring(qcChangeCount) .. " changes were made, leaving " .. tostring(qcInvalidCount - qcChangeCount) .. " invalid records remaining.")

end

function qcMergeSVData()

	for qcIndex, qcEntry in pairs(qcQuestDBv2) do
		table.insert(qcWorkingDB, qcIndex, qcEntry)
	end

	local qcNewCount = 0
	local qcRecordCount = 0
	local qcClassUpdate = 0
	local qcRaceUpdate = 0
	local qcFactionUpdate = 0
	local qcReputationUpdate = 0
	local qcPersonalMerge = true
	table.insert(qcWorkingLog, "---------------------------")
	table.insert(qcWorkingLog, "PROCESSING QUEST RECORDS...")
	table.insert(qcWorkingLog, "---------------------------")

	for qcIndex, qcEntry in pairs(qcSVData) do
		qcRecordCount = (qcRecordCount + 1)
		if (qcWorkingDB[qcIndex] == nil) then --[[ The quest is brand new ]]--
			table.insert(qcWorkingDB, qcIndex, qcEntry)
			--[[ For new quests, we set the class and race for "everyone", but leave faction as recorded ]]--
			--[[ The idea is that i will manually claw these back as needed, and if i clawed back too much, the update section of the below code will add the required classes back on ]]--
			qcWorkingDB[qcIndex][4][2] = 4095
			qcWorkingDB[qcIndex][4][3] = 1023
			table.insert(qcWorkingLog, "NEW QUEST FOUND: " .. tostring(qcEntry[1][2]) .. " [" .. tostring(qcEntry[1][1]) .. "]")
			qcNewCount = (qcNewCount + 1)
		else --[[ The quest already exists - start updates ]]--
			table.insert(qcWorkingLog, "EXISTING QUEST FOUND: " .. tostring(qcWorkingDB[qcIndex][1][2]) .. " [" .. tostring(qcWorkingDB[qcIndex][1][1]) .. "]")
			--[[ First we want to see if we have all the information, or just what was gathered via QUEST_DETAIL ]]--
			if (qcEntry[1][4] == "<Category Unknown>") then
				table.insert(qcWorkingLog, "    PRELIMINARY DATA FOUND - SOME FIELDS WILL NOT BE UPDATED")
				--[[ If for some random reason the ID was not present, update it ]]--
				if (qcEntry[1][1] == nil) then qcEntry[1][1] = qcIndex end
				--[[ Quest name will not be updated as it will remove any tags already present ]]--
				--[[ Quest level will not be updated as it can only be known after accepting the quest ]]--
				--[[ Quest category will not be updated as it can only be known after accepting the quest ]]--
				--[[ Quest type will not be updated as it will overwrite any manually set types (eg. repeatables or specials) ]]--
				if (qcWorkingDB[qcIndex][2][1] ~= qcEntry[2][1]) then
					if (qcEntry[2][1] ~= 99999) then
						table.insert(qcWorkingLog, "    ZONE ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][2][1]) .. "' NEW VALUE: '" .. tostring(qcEntry[2][1]) .. "'")
						qcWorkingDB[qcIndex][1][4] = qcEntry[2][1]
					end
				end
				if (qcWorkingDB[qcIndex][3][1] ~= qcEntry[3][1]) then
					if not (tostring(qcEntry[3][1]) == "0") then
						table.insert(qcWorkingLog, "    NPC ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][1]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][1]) .. "'")
					end
					qcWorkingDB[qcIndex][3][1] = qcEntry[3][1]
				end
				if (qcWorkingDB[qcIndex][3][2] ~= qcEntry[3][2]) then
					if not (tostring(qcEntry[3][2]) == "<Remote Quest Offer>") then
						table.insert(qcWorkingLog, "    NPC NAME UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][2]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][2]) .. "'")
					end
					qcWorkingDB[qcIndex][3][2] = qcEntry[3][2]
				end
				if (qcWorkingDB[qcIndex][3][3] ~= qcEntry[3][3]) then
					table.insert(qcWorkingLog, "    NPC ZONE UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][3]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][3]) .. "'")
					qcWorkingDB[qcIndex][3][3] = qcEntry[3][3]
				end
				if (qcWorkingDB[qcIndex][3][4] ~= qcEntry[3][4]) and (qcPersonalMerge) then
					table.insert(qcWorkingLog, "    NPC X UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][4]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][4]) .. "'")
					qcWorkingDB[qcIndex][3][4] = qcEntry[3][4]
				end
				if (qcWorkingDB[qcIndex][3][5] ~= qcEntry[3][5]) and (qcPersonalMerge) then
					table.insert(qcWorkingLog, "    NPC Y UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][5]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][5]) .. "'")
					qcWorkingDB[qcIndex][3][5] = qcEntry[3][5]
				end
				if (qcWorkingDB[qcIndex][3][6] ~= qcEntry[3][6]) then
					table.insert(qcWorkingLog, "    MAP ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][6]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][6]) .. "'")
					qcWorkingDB[qcIndex][3][6] = qcEntry[3][6]
				end
				if (qcWorkingDB[qcIndex][3][7] ~= qcEntry[3][7]) then
					table.insert(qcWorkingLog, "    MAP LEVEL UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][7]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][7]) .. "'")
					qcWorkingDB[qcIndex][3][7] = qcEntry[3][7]
				end
				--[[ Map ignore will not be updated as it will overwrite any manually set values ]]--
				--[[ Reputations will not be updated as it can only be known after accepting the quest ]]--
			else -- [[ The quest category is known, which means the quest must have been accepted, therefore all information should be available ]]--
				--[[ If for some random reason the ID was not present, update it ]]--
				if (qcEntry[1][1] == nil) then qcEntry[1][1] = qcIndex end
				if (qcWorkingDB[qcIndex][1][2] ~= qcEntry[1][2]) then
					--[[ Do a check to see if the first character of the quest name is a square bracket. If it is, don't use the data as it will likely have a level tag on the front of it ]]--
					if (string.sub(tostring(qcEntry[1][2]), 1, 1) ~= "[") then
						table.insert(qcWorkingLog, "    NAME UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][1][2]) .. "' NEW VALUE: '" .. tostring(qcEntry[1][2]) .. "'")
						qcWorkingDB[qcIndex][1][2] = qcEntry[1][2]
					end
				end
				if (qcWorkingDB[qcIndex][1][3] ~= qcEntry[1][3]) then
					table.insert(qcWorkingLog, "    LEVEL UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][1][3]) .. "' NEW VALUE: '" .. tostring(qcEntry[1][3]) .. "'")
					qcWorkingDB[qcIndex][1][3] = qcEntry[1][3]
				end
				if (qcWorkingDB[qcIndex][1][4] ~= qcEntry[1][4]) then
					table.insert(qcWorkingLog, "    CATEGORY UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][1][4]) .. "' NEW VALUE: '" .. tostring(qcEntry[1][4]) .. "'")
					qcWorkingDB[qcIndex][1][4] = qcEntry[1][4]
				end
				--[[ Quest type will not be updated as it will overwrite any manually set types (eg. repeatables or specials) ]]--
				if (qcWorkingDB[qcIndex][2][1] ~= qcEntry[2][1]) then
					if (qcEntry[2][1] ~= 99999) then
						table.insert(qcWorkingLog, "    ZONE ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][2][1]) .. "' NEW VALUE: '" .. tostring(qcEntry[2][1]) .. "'")
						qcWorkingDB[qcIndex][1][4] = qcEntry[2][1]
					end
				end
				if (qcWorkingDB[qcIndex][3][1] ~= qcEntry[3][1]) then
					if not (tostring(qcEntry[3][1]) == "0") then
						table.insert(qcWorkingLog, "    NPC ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][1]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][1]) .. "'")
					end
					qcWorkingDB[qcIndex][3][1] = qcEntry[3][1]
				end
				if (qcWorkingDB[qcIndex][3][2] ~= qcEntry[3][2]) then
					if not (tostring(qcEntry[3][2]) == "<Remote Quest Offer>") then
						table.insert(qcWorkingLog, "    NPC NAME UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][2]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][2]) .. "'")
					end
					qcWorkingDB[qcIndex][3][2] = qcEntry[3][2]
				end
				if (qcWorkingDB[qcIndex][3][3] ~= qcEntry[3][3]) then
					table.insert(qcWorkingLog, "    NPC ZONE UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][3]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][3]) .. "'")
					qcWorkingDB[qcIndex][3][3] = qcEntry[3][3]
				end
				if (qcWorkingDB[qcIndex][3][4] ~= qcEntry[3][4]) and (qcPersonalMerge) then
					table.insert(qcWorkingLog, "    NPC X UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][4]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][4]) .. "'")
					qcWorkingDB[qcIndex][3][4] = qcEntry[3][4]
				end
				if (qcWorkingDB[qcIndex][3][5] ~= qcEntry[3][5]) and (qcPersonalMerge) then
					table.insert(qcWorkingLog, "    NPC Y UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][5]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][5]) .. "'")
					qcWorkingDB[qcIndex][3][5] = qcEntry[3][5]
				end
				if (qcWorkingDB[qcIndex][3][6] ~= qcEntry[3][6]) then
					table.insert(qcWorkingLog, "    MAP ID UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][6]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][6]) .. "'")
					qcWorkingDB[qcIndex][3][6] = qcEntry[3][6]
				end
				if (qcWorkingDB[qcIndex][3][7] ~= qcEntry[3][7]) then
					table.insert(qcWorkingLog, "    MAP LEVEL UPDATE - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][3][7]) .. "' NEW VALUE: '" .. tostring(qcEntry[3][7]) .. "'")
					qcWorkingDB[qcIndex][3][7] = qcEntry[3][7]
				end
				--[[ Map ignore will not be updated as it will overwrite any manually set values ]]--
				--[[ Faction Merge Code ]]--
				do
					local qcEntryFaction = qcEntry[4][1]
					local qcExistingFaction = qcWorkingDB[qcIndex][4][1]
					local qcNewFaction = 0

					local qcEntryAlliance, qcEntryHorde = 0, 0
					local qcExistingAlliance, qcExistingHorde = 0, 0
					local qcNewAlliance, qcNewHorde = 0, 0

					if (bit.band(qcEntryFaction, qcFactionBits["ALLIANCE"]) ~= 0) then qcEntryAlliance = qcFactionBits["ALLIANCE"] end
					if (bit.band(qcEntryFaction, qcFactionBits["HORDE"]) ~= 0) then qcEntryHorde = qcFactionBits["HORDE"] end

					if (bit.band(qcExistingFaction, qcFactionBits["ALLIANCE"]) ~= 0) then qcExistingAlliance = qcFactionBits["ALLIANCE"] end
					if (bit.band(qcExistingFaction, qcFactionBits["HORDE"]) ~= 0) then qcExistingHorde = qcFactionBits["HORDE"] end

					if (qcEntryAlliance > qcExistingAlliance) then
						table.insert(qcWorkingLog, "    FACTION UPDATE - ALLIANCE ADDED")
						qcFactionUpdate = (qcFactionUpdate + 1)
					end
					qcNewAlliance = math.max(qcEntryAlliance, qcExistingAlliance)
					if (qcEntryHorde > qcExistingHorde) then
						table.insert(qcWorkingLog, "    FACTION UPDATE - HORDE ADDED")
						qcFactionUpdate = (qcFactionUpdate + 1)
					end
					qcNewHorde = math.max(qcEntryHorde, qcExistingHorde)
					qcNewFaction = (qcNewAlliance + qcNewHorde)
					qcWorkingDB[qcIndex][4][1] = qcNewFaction
				end
				--[[ Race Merge Code ]]--
				do
					local qcEntryRace = qcEntry[4][2]
					local qcExistingRace = qcWorkingDB[qcIndex][4][2]
					local qcNewRace = 0

					local qcEntryHuman, qcEntryOrc, qcEntryDwarf, qcEntryNightelf, qcEntryScourge, qcEntryTauren, qcEntryGnome, qcEntryTroll, qcEntryGoblin, qcEntryBloodelf, qcEntryDraenei, qcEntryWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcExistingHuman, qcExistingOrc, qcExistingDwarf, qcExistingNightelf, qcExistingScourge, qcExistingTauren, qcExistingGnome, qcExistingTroll, qcExistingGoblin, qcExistingBloodelf, qcExistingDraenei, qcExistingWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcNewHuman, qcNewOrc, qcNewDwarf, qcNewNightelf, qcNewScourge, qcNewTauren, qcNewGnome, qcNewTroll, qcNewGoblin, qcNewBloodelf, qcNewDraenei, qcNewWorgen = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

					if (bit.band(qcEntryRace, qcRaceBits["HUMAN"]) ~= 0) then qcEntryHuman = qcRaceBits["HUMAN"] end
					if (bit.band(qcEntryRace, qcRaceBits["ORC"]) ~= 0) then qcEntryOrc = qcRaceBits["ORC"] end
					if (bit.band(qcEntryRace, qcRaceBits["DWARF"]) ~= 0) then qcEntryDwarf = qcRaceBits["DWARF"] end
					if (bit.band(qcEntryRace, qcRaceBits["NIGHTELF"]) ~= 0) then qcEntryNightelf = qcRaceBits["NIGHTELF"] end
					if (bit.band(qcEntryRace, qcRaceBits["SCOURGE"]) ~= 0) then qcEntryScourge = qcRaceBits["SCOURGE"] end
					if (bit.band(qcEntryRace, qcRaceBits["TAUREN"]) ~= 0) then qcEntryTauren = qcRaceBits["TAUREN"] end
					if (bit.band(qcEntryRace, qcRaceBits["GNOME"]) ~= 0) then qcEntryGnome = qcRaceBits["GNOME"] end
					if (bit.band(qcEntryRace, qcRaceBits["TROLL"]) ~= 0) then qcEntryTroll = qcRaceBits["TROLL"] end
					if (bit.band(qcEntryRace, qcRaceBits["GOBLIN"]) ~= 0) then qcEntryGoblin = qcRaceBits["GOBLIN"] end
					if (bit.band(qcEntryRace, qcRaceBits["BLOODELF"]) ~= 0) then qcEntryBloodelf = qcRaceBits["BLOODELF"] end
					if (bit.band(qcEntryRace, qcRaceBits["DRAENEI"]) ~= 0) then qcEntryDraenei = qcRaceBits["DRAENEI"] end
					if (bit.band(qcEntryRace, qcRaceBits["WORGEN"]) ~= 0) then qcEntryWorgen = qcRaceBits["WORGEN"] end

					if (bit.band(qcExistingRace, qcRaceBits["HUMAN"]) ~= 0) then qcExistingHuman = qcRaceBits["HUMAN"] end
					if (bit.band(qcExistingRace, qcRaceBits["ORC"]) ~= 0) then qcExistingOrc = qcRaceBits["ORC"] end
					if (bit.band(qcExistingRace, qcRaceBits["DWARF"]) ~= 0) then qcExistingDwarf = qcRaceBits["DWARF"] end
					if (bit.band(qcExistingRace, qcRaceBits["NIGHTELF"]) ~= 0) then qcExistingNightelf = qcRaceBits["NIGHTELF"] end
					if (bit.band(qcExistingRace, qcRaceBits["SCOURGE"]) ~= 0) then qcExistingScourge = qcRaceBits["SCOURGE"] end
					if (bit.band(qcExistingRace, qcRaceBits["TAUREN"]) ~= 0) then qcExistingTauren = qcRaceBits["TAUREN"] end
					if (bit.band(qcExistingRace, qcRaceBits["GNOME"]) ~= 0) then qcExistingGnome = qcRaceBits["GNOME"] end
					if (bit.band(qcExistingRace, qcRaceBits["TROLL"]) ~= 0) then qcExistingTroll = qcRaceBits["TROLL"] end
					if (bit.band(qcExistingRace, qcRaceBits["GOBLIN"]) ~= 0) then qcExistingGoblin = qcRaceBits["GOBLIN"] end
					if (bit.band(qcExistingRace, qcRaceBits["BLOODELF"]) ~= 0) then qcExistingBloodelf = qcRaceBits["BLOODELF"] end
					if (bit.band(qcExistingRace, qcRaceBits["DRAENEI"]) ~= 0) then qcExistingDraenei = qcRaceBits["DRAENEI"] end
					if (bit.band(qcExistingRace, qcRaceBits["WORGEN"]) ~= 0) then qcExistingWorgen = qcRaceBits["WORGEN"] end

					if (qcEntryHuman > qcExistingHuman) then
						table.insert(qcWorkingLog, "    RACE UPDATE - HUMAN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewHuman = math.max(qcEntryHuman, qcExistingHuman)
					if (qcEntryOrc > qcExistingOrc) then
						table.insert(qcWorkingLog, "    RACE UPDATE - ORC ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewOrc = math.max(qcEntryOrc, qcExistingOrc)
					if (qcEntryDwarf > qcExistingDwarf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - DWARF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewDwarf = math.max(qcEntryDwarf, qcExistingDwarf)
					if (qcEntryNightelf > qcExistingNightelf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - NIGHTELF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewNightelf = math.max(qcEntryNightelf, qcExistingNightelf)
					if (qcEntryScourge > qcExistingScourge) then
						table.insert(qcWorkingLog, "    RACE UPDATE - SCOURGE ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewScourge = math.max(qcEntryScourge, qcExistingScourge)
					if (qcEntryTauren > qcExistingTauren) then
						table.insert(qcWorkingLog, "    RACE UPDATE - TAUREN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewTauren = math.max(qcEntryTauren, qcExistingTauren)
					if (qcEntryGnome > qcExistingGnome) then
						table.insert(qcWorkingLog, "    RACE UPDATE - GNOME ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewGnome = math.max(qcEntryGnome, qcExistingGnome)
					if (qcEntryTroll > qcExistingTroll) then
						table.insert(qcWorkingLog, "    RACE UPDATE - TROLL ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewTroll = math.max(qcEntryTroll, qcExistingTroll)
					if (qcEntryGoblin > qcExistingGoblin) then
						table.insert(qcWorkingLog, "    RACE UPDATE - GOBLIN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewGoblin = math.max(qcEntryGoblin, qcExistingGoblin)
					if (qcEntryBloodelf > qcExistingBloodelf) then
						table.insert(qcWorkingLog, "    RACE UPDATE - BLOODELF ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewBloodelf = math.max(qcEntryBloodelf, qcExistingBloodelf)
					if (qcEntryDraenei > qcExistingDraenei) then
						table.insert(qcWorkingLog, "    RACE UPDATE - DRAENEI ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewDraenei = math.max(qcEntryDraenei, qcExistingDraenei)
					if (qcEntryWorgen > qcExistingWorgen) then
						table.insert(qcWorkingLog, "    RACE UPDATE - WORGEN ADDED")
						qcRaceUpdate = (qcRaceUpdate + 1)
					end
					qcNewWorgen = math.max(qcEntryWorgen, qcExistingWorgen)
					qcNewRace = (qcNewHuman + qcNewOrc + qcNewDwarf + qcNewNightelf + qcNewScourge + qcNewTauren + qcNewGnome + qcNewTroll + qcNewGoblin + qcNewBloodelf + qcNewDraenei + qcNewWorgen)
					qcWorkingDB[qcIndex][4][2] = qcNewRace
				end
				--[[ Class Merge Code ]]--
				do
					local qcEntryClass = qcEntry[4][3]
					local qcExistingClass = qcWorkingDB[qcIndex][4][3]
					local qcNewClass = 0

					local qcEntryWarrior, qcEntryPaladin, qcEntryHunter, qcEntryRogue, qcEntryPriest, qcEntryDeathknight, qcEntryShaman, qcEntryMage, qcEntryWarlock, qcEntryDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcExistingWarrior, qcExistingPaladin, qcExistingHunter, qcExistingRogue, qcExistingPriest, qcExistingDeathknight, qcExistingShaman, qcExistingMage, qcExistingWarlock, qcExistingDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
					local qcNewWarrior, qcNewPaladin, qcNewHunter, qcNewRogue, qcNewPriest, qcNewDeathknight, qcNewShaman, qcNewMage, qcNewWarlock, qcNewDruid = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

					if (bit.band(qcEntryClass, qcClassBits["WARRIOR"]) ~= 0) then qcEntryWarrior = qcClassBits["WARRIOR"] end
					if (bit.band(qcEntryClass, qcClassBits["PALADIN"]) ~= 0) then qcEntryPaladin = qcClassBits["PALADIN"] end
					if (bit.band(qcEntryClass, qcClassBits["HUNTER"]) ~= 0) then qcEntryHunter = qcClassBits["HUNTER"] end
					if (bit.band(qcEntryClass, qcClassBits["ROGUE"]) ~= 0) then qcEntryRogue = qcClassBits["ROGUE"] end
					if (bit.band(qcEntryClass, qcClassBits["PRIEST"]) ~= 0) then qcEntryPriest = qcClassBits["PRIEST"] end
					if (bit.band(qcEntryClass, qcClassBits["DEATHKNIGHT"]) ~= 0) then qcEntryDeathknight = qcClassBits["DEATHKNIGHT"] end
					if (bit.band(qcEntryClass, qcClassBits["SHAMAN"]) ~= 0) then qcEntryShaman = qcClassBits["SHAMAN"] end
					if (bit.band(qcEntryClass, qcClassBits["MAGE"]) ~= 0) then qcEntryMage = qcClassBits["MAGE"] end
					if (bit.band(qcEntryClass, qcClassBits["WARLOCK"]) ~= 0) then qcEntryWarlock = qcClassBits["WARLOCK"] end
					if (bit.band(qcEntryClass, qcClassBits["DRUID"]) ~= 0) then qcEntryDruid = qcClassBits["DRUID"] end

					if (bit.band(qcExistingClass, qcClassBits["WARRIOR"]) ~= 0) then qcExistingWarrior = qcClassBits["WARRIOR"] end
					if (bit.band(qcExistingClass, qcClassBits["PALADIN"]) ~= 0) then qcExistingPaladin = qcClassBits["PALADIN"] end
					if (bit.band(qcExistingClass, qcClassBits["HUNTER"]) ~= 0) then qcExistingHunter = qcClassBits["HUNTER"] end
					if (bit.band(qcExistingClass, qcClassBits["ROGUE"]) ~= 0) then qcExistingRogue = qcClassBits["ROGUE"] end
					if (bit.band(qcExistingClass, qcClassBits["PRIEST"]) ~= 0) then qcExistingPriest = qcClassBits["PRIEST"] end
					if (bit.band(qcExistingClass, qcClassBits["DEATHKNIGHT"]) ~= 0) then qcExistingDeathknight = qcClassBits["DEATHKNIGHT"] end
					if (bit.band(qcExistingClass, qcClassBits["SHAMAN"]) ~= 0) then qcExistingShaman = qcClassBits["SHAMAN"] end
					if (bit.band(qcExistingClass, qcClassBits["MAGE"]) ~= 0) then qcExistingMage = qcClassBits["MAGE"] end
					if (bit.band(qcExistingClass, qcClassBits["WARLOCK"]) ~= 0) then qcExistingWarlock = qcClassBits["WARLOCK"] end
					if (bit.band(qcExistingClass, qcClassBits["DRUID"]) ~= 0) then qcExistingDruid = qcClassBits["DRUID"] end

					if (qcEntryWarrior > qcExistingWarrior) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - WARRIOR ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewWarrior = math.max(qcEntryWarrior, qcExistingWarrior)
					if (qcEntryPaladin > qcExistingPaladin) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - PALADIN ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewPaladin = math.max(qcEntryPaladin, qcExistingPaladin)
					if (qcEntryHunter > qcExistingHunter) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - HUNTER ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewHunter = math.max(qcEntryHunter, qcExistingHunter)
					if (qcEntryRogue > qcExistingRogue) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - ROGUE ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewRogue = math.max(qcEntryRogue, qcExistingRogue)
					if (qcEntryPriest > qcExistingPriest) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - PRIEST ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewPriest = math.max(qcEntryPriest, qcExistingPriest)
					if (qcEntryDeathknight > qcExistingDeathknight) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - DEATHKNIGHT ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewDeathknight = math.max(qcEntryDeathknight, qcExistingDeathknight)
					if (qcEntryShaman > qcExistingShaman) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - SHAMAN ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewShaman = math.max(qcEntryShaman, qcExistingShaman)
					if (qcEntryMage > qcExistingMage) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - MAGE ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewMage = math.max(qcEntryMage, qcExistingMage)
					if (qcEntryWarlock > qcExistingWarlock) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - WARLOCK ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewWarlock = math.max(qcEntryWarlock, qcExistingWarlock)
					if (qcEntryDruid > qcExistingDruid) then
						table.insert(qcWorkingLog, "    CLASS UPDATE - DRUID ADDED")
						qcClassUpdate = (qcClassUpdate + 1)
					end
					qcNewDruid = math.max(qcEntryDruid, qcExistingDruid)
					qcNewClass = (qcNewWarrior + qcNewPaladin + qcNewHunter + qcNewRogue + qcNewPriest + qcNewDeathknight + qcNewShaman + qcNewMage + qcNewWarlock + qcNewDruid)
					qcWorkingDB[qcIndex][4][3] = qcNewClass
				end
				--[[ Reputation Merge Code ]]--
				do
					if (qcEntry[6] ~= nil) then
						if (qcWorkingDB[qcIndex][6] == nil) then qcWorkingDB[qcIndex][6] = {} end
						for qcReputationIndex, qcReputationEntry in pairs(qcEntry[6]) do
							qcReputationUpdate = (qcReputationUpdate + 1)
							if (qcWorkingDB[qcIndex][6][qcReputationIndex] == nil) then
								table.insert(qcWorkingLog, "    REPUTATION UPDATE - NEW FACTION: '" .. tostring(qcReputationIndex) .. "' WITH '" .. tostring(qcReputationEntry) .. "' REPUTATION GAIN")
								qcWorkingDB[qcIndex][6][qcReputationIndex] = qcReputationEntry
							else
								if (qcWorkingDB[qcIndex][6][qcReputationIndex] ~= qcReputationEntry) then
									table.insert(qcWorkingLog, "    REPUTATION VALUE UPDATE FOR " .. tostring(qcReputationIndex) .. " - ORIGINAL VALUE: '" .. tostring(qcWorkingDB[qcIndex][6][qcReputationIndex]) .. "' NEW VALUE: '" .. tostring(qcReputationEntry) .. "'")
									qcWorkingDB[qcIndex][6][qcReputationIndex] = qcReputationEntry
								end
							end
						end
					end
				end
			end
		end
	end

	print(tostring(qcRecordCount) .. " quest records checked and updated. " .. tostring(qcNewCount) .. " brand new quests added. " .. tostring(qcClassUpdate) .. " class bitwise updates made, " .. tostring(qcRaceUpdate) .. " race bitwise updates made, " .. tostring(qcFactionUpdate) .. " faction bitwise updates made and " .. tostring(qcReputationUpdate) .. " reputation updates made.")

end

local function qcGetQuestCategory(qcQuestID)

local _S
local qcIsHeader
local qcHeader
local qcQuestIDIndex = GetQuestLogIndexByID(qcQuestID)

	if (qcQuestIDIndex == 0) then return nil end

	while (qcQuestIDIndex >= 1) do
		qcHeader, _S, _S, _S, qcIsHeader, _S, _S, _S, _S = GetQuestLogTitle(qcQuestIDIndex)
		if (qcIsHeader == 1) then return qcHeader end
		qcQuestIDIndex = qcQuestIDIndex - 1
	end

	return nil

end

function qcSetClassBits()

	local qcChangeCount = 0
	local qcRecordCount = 0
	table.insert(qcWorkingLog, "--------------------------")
	table.insert(qcWorkingLog, "PROCESSING CLASS QUESTS...")
	table.insert(qcWorkingLog, "--------------------------")

	for qcIndex, qcEntry in pairs(qcWorkingDB) do
		qcRecordCount = (qcRecordCount + 1)
		if (string.upper(qcEntry[1][4]) == "WARRIOR") then
			if (qcEntry[4][3] ~= 1) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 1
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR WARRIOR")
			end
		elseif (string.upper(qcEntry[1][4]) == "PALADIN") then
			if (qcEntry[4][3] ~= 2) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 2
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR PALADIN")
			end
		elseif (string.upper(qcEntry[1][4]) == "HUNTER") then
			if (qcEntry[4][3] ~= 4) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 4
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR HUNTER")
			end
		elseif (string.upper(qcEntry[1][4]) == "ROGUE") then
			if (qcEntry[4][3] ~= 8) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 8
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR ROGUE")
			end
		elseif (string.upper(qcEntry[1][4]) == "PRIEST") then
			if (qcEntry[4][3] ~= 16) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 16
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR PRIEST")
			end
		elseif (string.upper(qcEntry[1][4]) == "DEATHKNIGHT") then
			if (qcEntry[4][3] ~= 32) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 32
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR DEATHKNIGHT")
			end
		elseif (string.upper(qcEntry[1][4]) == "SHAMAN") then
			if (qcEntry[4][3] ~= 64) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 64
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR SHAMAN")
			end
		elseif (string.upper(qcEntry[1][4]) == "MAGE") then
			if (qcEntry[4][3] ~= 128) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 128
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR MAGE")
			end
		elseif (string.upper(qcEntry[1][4]) == "WARLOCK") then
			if (qcEntry[4][3] ~= 256) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 256
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR WARLOCK")
			end
		elseif (string.upper(qcEntry[1][4]) == "DRUID") then
			if (qcEntry[4][3] ~= 512) then
				qcChangeCount = (qcChangeCount + 1)
				qcEntry[4][3] = 512
				table.insert(qcWorkingLog, "CLASS QUEST UPDATE: " .. tostring(qcEntry[1][2]) .. " HAS BEEN FLAGGED FOR DRUID")
			end
		end
	end

	print(tostring(qcRecordCount) .. " records checked against class categories and updated. " .. tostring(qcChangeCount) .. " changes were made.")

end

local function qcRound(num, idp)

	return tonumber(string.format("%." .. (idp or 0) .. "f", num))

end

function qcCollectOnProgress()
if (GetLocale() == "enUS") or (GetLocale() == "enGB") then

	local qcQuestID = GetQuestID()
	if (qcQuestID == 0) or (qcQuestID == nil) then return nil end

	if (qcCollected[qcQuestID] ~= nil) then
		--print("QUEST_PROGRESS: Quest already exists in DB. Not likely to be repeatable or seasonal.")
		return nil
	else
		--print("QUEST_PROGRESS: Quest does not already exist in DB meaning it is likely to be repeatable, seasonal, or a quest that was in the players log before installing the addon.")
		--[[ TODO: Upon addon's first load, check quest log to record (or create blank records) for all quests ]]--

		local qcQuestName = GetTitleText()

		if (QuestIsDaily()) then return nil end --[[ Can't be flagged repeatable if daily ]]--
		if (QuestIsWeekly()) then return nil end --[[ Can't be flagged repeatable if weekly ]]--

		local qcNPCID = 0
		local qcNPCName = UnitName("questnpc")
		if (UnitExists("questnpc")) then
			if (qcNPCName == UnitName("player")) then
				qcNPCName = "<Remote Quest Offer>"
			elseif (UnitInParty(qcNPCName) == 1) then
				print("|cFF9482C9Quest Completist:|r This quest appears to be shared from someone in your party. Not recording the quest information.")
				table.insert(qcSharedIDs, qcQuestID, 1)
				return nil --[[ Appears to be shared, not recording quest ]]--
			elseif (UnitInRaid(qcNPCName) == 1) then
				print("|cFF9482C9Quest Completist:|r This quest appears to be shared from someone in your raid. Not recording the quest information.")
				table.insert(qcSharedIDs, qcQuestID, 1)
				return nil --[[ Appears to be shared, not recording quest ]]--
			else
				qcNPCID = tonumber(strsub(UnitGUID("questnpc"), 7, 10), 16)
			end
		end

		local qcNPCZone = GetRealZoneText()

		local qcNPCX = 0
		local qcNPCY = 0
		do
			local qcX, qcY = GetPlayerMapPosition("player")
			if (qcX > 0) and (qcY > 0) then
				qcNPCX = qcRound(qcX*100, 1)
				qcNPCY = qcRound(qcY*100, 1)
			end
		end

		--[[ TODO: Possible use of GetQuestWorldMapAreaID? ]]--
		local qcMapID = 0
		local qcMapLevel = 0
		do
			SetMapToCurrentZone()
			if (GetCurrentMapAreaID() > 0) then
				qcMapID = GetCurrentMapAreaID()
			end
			if (GetCurrentMapDungeonLevel() > 0) then
				qcMapLevel = GetCurrentMapDungeonLevel()
			end
		end

		local qcFaction = 0
		do
			local qcEFaction, _S = UnitFactionGroup("player")
			qcFaction = qcFactionBits[string.upper(qcEFaction)]
		end

		local qcRace = 0
		do
			local _S, qcERace = UnitRace("player")
			qcRace = qcRaceBits[string.upper(qcERace)]
		end

		local qcClass = 0
		do
			local _S, qcEClass = UnitClass("player")
			qcClass = qcClassBits[string.upper(qcEClass)]
		end

		if (qcRepeatables[qcQuestID] == nil) then
			qcRepeatables[qcQuestID] = {}
			qcRepeatables[qcQuestID][1] = {}
			qcRepeatables[qcQuestID][3] = {}
			qcRepeatables[qcQuestID][4] = {}
			qcRepeatables[qcQuestID][1][1] = qcQuestID
			qcRepeatables[qcQuestID][1][2] = qcQuestName
			qcRepeatables[qcQuestID][3][1] = qcNPCID
			qcRepeatables[qcQuestID][3][2] = qcNPCName
			qcRepeatables[qcQuestID][3][3] = qcNPCZone
			qcRepeatables[qcQuestID][3][4] = qcNPCX
			qcRepeatables[qcQuestID][3][5] = qcNPCY
			qcRepeatables[qcQuestID][3][6] = qcMapID
			qcRepeatables[qcQuestID][3][7] = qcMapLevel
			qcRepeatables[qcQuestID][4][1] = qcFaction
			qcRepeatables[qcQuestID][4][2] = qcRace
			qcRepeatables[qcQuestID][4][3] = qcClass
		else
			if (qcRepeatables[qcQuestID][1] == nil) then qcRepeatables[qcQuestID][1] = {} end
			if (qcRepeatables[qcQuestID][3] == nil) then qcRepeatables[qcQuestID][3] = {} end
			if (qcRepeatables[qcQuestID][4] == nil) then qcRepeatables[qcQuestID][4] = {} end
			qcRepeatables[qcQuestID][1][1] = qcQuestID
			qcRepeatables[qcQuestID][1][2] = qcQuestName
			qcRepeatables[qcQuestID][3][1] = qcNPCID
			qcRepeatables[qcQuestID][3][2] = qcNPCName
			qcRepeatables[qcQuestID][3][3] = qcNPCZone
			qcRepeatables[qcQuestID][3][4] = qcNPCX
			qcRepeatables[qcQuestID][3][5] = qcNPCY
			qcRepeatables[qcQuestID][3][6] = qcMapID
			qcRepeatables[qcQuestID][3][7] = qcMapLevel
			if (bit.band(qcRepeatables[qcQuestID][4][1], qcFaction) == 0) then
				qcRepeatables[qcQuestID][4][1] = tonumber(qcRepeatables[qcQuestID][4][1] + qcFaction)
			end
			if (bit.band(qcRepeatables[qcQuestID][4][2], qcRace) == 0) then
				qcRepeatables[qcQuestID][4][2] = tonumber(qcRepeatables[qcQuestID][4][2] + qcRace)
			end
			if (bit.band(qcRepeatables[qcQuestID][4][3], qcClass) == 0) then
				qcRepeatables[qcQuestID][4][3] = tonumber(qcRepeatables[qcQuestID][4][3] + qcClass)
			end
		end
	end

end
end

function qcCollectOnDetail()
if (GetLocale() == "enUS") or (GetLocale() == "enGB") then

	local qcQuestID = GetQuestID()
	if (qcQuestID == 0) or (qcQuestID == nil) then return nil end

	local qcQuestName = GetTitleText()
	local qcQuestLevel = 0
	--[[ ON ACCEPT
	do
		local _S, qcN, qcL, qcT
		qcN, qcL, qcT, _S, _S, _S, _S, _S, _S = GetQuestLogTitle(GetQuestLogIndexByID(qcQuestID))
		if (qcT == nil) then qcQuestName = tostring(qcN) else qcQuestName = tostring(qcN .. " [" .. qcT .. "]") end
		if (qcL > 0) then qcQuestLevel = qcL end
	end
	]]--

	--[[ ON ACCEPT
	local qcCategory = (qcGetQuestCategory(qcQuestID) or "<Category Unknown>")
	--]]
	local qcCategory = "<Category Unknown>"

	local qcQuestType = 1
	if (QuestIsDaily()) then
		qcQuestType = 3
	elseif (QuestIsWeekly()) then
		qcQuestType = 5
	end

	local qcZoneID = 99999
	--[[ ON ACCEPT
	for qcIndex, qcEntry in ipairs(qcQuestCategories) do
		if (qcEntry[2] == qcCategory) then
			qcZoneID = qcEntry[1]
			break
		end
	end
	--]]

	local qcNPCID = 0
	local qcNPCName = UnitName("questnpc")
	if (UnitExists("questnpc")) then
		if (qcNPCName == UnitName("player")) then
			qcNPCName = "<Remote Quest Offer>"
		elseif (UnitInParty(qcNPCName) == 1) then
			print("|cFF9482C9Quest Completist:|r This quest appears to be shared from someone in your party. Not recording the quest information.")
			table.insert(qcSharedIDs, qcQuestID, 1)
			return nil --[[ Appears to be shared, not recording quest ]]--
		elseif (UnitInRaid(qcNPCName) == 1) then
			print("|cFF9482C9Quest Completist:|r This quest appears to be shared from someone in your raid. Not recording the quest information.")
			table.insert(qcSharedIDs, qcQuestID, 1)
			return nil --[[ Appears to be shared, not recording quest ]]--
		else
			qcNPCID = tonumber(strsub(UnitGUID("questnpc"), 7, 10), 16)
		end
	end

	local qcNPCZone = GetRealZoneText()

	local qcNPCX = 0
	local qcNPCY = 0
	do
		local qcX, qcY = GetPlayerMapPosition("player")
		if (qcX > 0) and (qcY > 0) then
			qcNPCX = qcRound(qcX*100, 1)
			qcNPCY = qcRound(qcY*100, 1)
		end
	end

	--[[ TODO: Possible use of GetQuestWorldMapAreaID? ]]--
	local qcMapID = 0
	local qcMapLevel = 0
	do
		SetMapToCurrentZone()
		if (GetCurrentMapAreaID() > 0) then
			qcMapID = GetCurrentMapAreaID()
		end
		if (GetCurrentMapDungeonLevel() > 0) then
			qcMapLevel = GetCurrentMapDungeonLevel()
		end
	end

	local qcMapIgnore = 0

	local qcFaction = 0
	do
		local qcEFaction, _S = UnitFactionGroup("player")
		qcFaction = qcFactionBits[string.upper(qcEFaction)]
	end

	local qcRace = 0
	do
		local _S, qcERace = UnitRace("player")
		qcRace = qcRaceBits[string.upper(qcERace)]
	end

	local qcClass = 0
	do
		local _S, qcEClass = UnitClass("player")
		qcClass = qcClassBits[string.upper(qcEClass)]
	end

	local qcQuestNotes = nil

	if (qcCollected[qcQuestID] == nil) then
		qcCollected[qcQuestID] = {}
		qcCollected[qcQuestID][1] = {}
		qcCollected[qcQuestID][2] = {}
		qcCollected[qcQuestID][3] = {}
		qcCollected[qcQuestID][4] = {}
		qcCollected[qcQuestID][5] = {}
		qcCollected[qcQuestID][6] = {}
		qcCollected[qcQuestID][1][1] = qcQuestID
		qcCollected[qcQuestID][1][2] = qcQuestName
		qcCollected[qcQuestID][1][3] = qcQuestLevel
		qcCollected[qcQuestID][1][4] = qcCategory
		qcCollected[qcQuestID][1][5] = qcQuestType
		qcCollected[qcQuestID][2][1] = qcZoneID
		qcCollected[qcQuestID][3][1] = qcNPCID
		qcCollected[qcQuestID][3][2] = qcNPCName
		qcCollected[qcQuestID][3][3] = qcNPCZone
		qcCollected[qcQuestID][3][4] = qcNPCX
		qcCollected[qcQuestID][3][5] = qcNPCY
		qcCollected[qcQuestID][3][6] = qcMapID
		qcCollected[qcQuestID][3][7] = qcMapLevel
		qcCollected[qcQuestID][3][8] = qcMapIgnore
		qcCollected[qcQuestID][4][1] = qcFaction
		qcCollected[qcQuestID][4][2] = qcRace
		qcCollected[qcQuestID][4][3] = qcClass
		qcCollected[qcQuestID][5][1] = qcQuestNotes
	else
		if (qcCollected[qcQuestID][1] == nil) then qcCollected[qcQuestID][1] = {} end
		if (qcCollected[qcQuestID][2] == nil) then qcCollected[qcQuestID][2] = {} end
		if (qcCollected[qcQuestID][3] == nil) then qcCollected[qcQuestID][3] = {} end
		if (qcCollected[qcQuestID][4] == nil) then qcCollected[qcQuestID][4] = {} end
		if (qcCollected[qcQuestID][5] == nil) then qcCollected[qcQuestID][5] = {} end
		if (qcCollected[qcQuestID][6] == nil) then qcCollected[qcQuestID][6] = {} end
		qcCollected[qcQuestID][1][1] = qcQuestID
		qcCollected[qcQuestID][1][2] = qcQuestName
		qcCollected[qcQuestID][1][3] = qcQuestLevel
		qcCollected[qcQuestID][1][4] = qcCategory
		qcCollected[qcQuestID][1][5] = qcQuestType
		qcCollected[qcQuestID][2][1] = qcZoneID
		qcCollected[qcQuestID][3][1] = qcNPCID
		qcCollected[qcQuestID][3][2] = qcNPCName
		qcCollected[qcQuestID][3][3] = qcNPCZone
		qcCollected[qcQuestID][3][4] = qcNPCX
		qcCollected[qcQuestID][3][5] = qcNPCY
		qcCollected[qcQuestID][3][6] = qcMapID
		qcCollected[qcQuestID][3][7] = qcMapLevel
		qcCollected[qcQuestID][3][8] = qcMapIgnore
		if (bit.band(qcCollected[qcQuestID][4][1], qcFaction) == 0) then
			qcCollected[qcQuestID][4][1] = tonumber(qcCollected[qcQuestID][4][1] + qcFaction)
		end
		if (bit.band(qcCollected[qcQuestID][4][2], qcRace) == 0) then
			qcCollected[qcQuestID][4][2] = tonumber(qcCollected[qcQuestID][4][2] + qcRace)
		end
		if (bit.band(qcCollected[qcQuestID][4][3], qcClass) == 0) then
			qcCollected[qcQuestID][4][3] = tonumber(qcCollected[qcQuestID][4][3] + qcClass)
		end
		qcCollected[qcQuestID][5][1] = qcQuestNotes
	end

	--[[ Show quest ID on the quest detail frame ]]--
	local qcNPCNameText = QuestFrameNpcNameText:GetText()
	QuestFrameNpcNameText:SetText((qcNPCNameText or "#") .. " [" .. tostring(qcQuestID) .. "]")

end
end

function qcCollectOnAccept(...)
if (GetLocale() == "enUS") or (GetLocale() == "enGB") then

	local qcQuestLogIndex, qcQuestID = ... --[[ Passed on through QUEST_ACCEPT event ]]--
	if (qcQuestID == 0) or (qcQuestID == nil) then return nil end

	local qcQuestName --[[ Collected via quest log with any tags attached ]]--
	local qcQuestLevel = 0
	do
		local _S, qcN, qcL, qcT
		qcN, qcL, qcT, _S, _S, _S, _S, _S, _S = GetQuestLogTitle(qcQuestLogIndex)
		if (qcT == nil) then qcQuestName = tostring(qcN) else qcQuestName = tostring(qcN .. " [" .. qcT .. "]") end
		if (qcL > 0) then qcQuestLevel = qcL end
	end

	local qcCategory = (qcGetQuestCategory(qcQuestID) or "<Category Unknown>")

	local qcQuestType = 1
	if (QuestIsDaily()) then
		qcQuestType = 3
	elseif (QuestIsWeekly()) then
		qcQuestType = 5
	end

	local qcZoneID = 99999
	for qcIndex, qcEntry in ipairs(qcQuestCategories) do
		if (qcEntry[2] == qcCategory) then
			qcZoneID = qcEntry[1]
			break
		end
	end

	if (qcSharedIDs[qcQuestID] ~= nil) then
		return nil --[[ Appears to be shared, not recording quest ]]--
	end

	local qcNPCZone = GetRealZoneText()

	local qcNPCX = 0
	local qcNPCY = 0
	do
		local qcX, qcY = GetPlayerMapPosition("player")
		if (qcX > 0) and (qcY > 0) then
			qcNPCX = qcRound(qcX*100, 1)
			qcNPCY = qcRound(qcY*100, 1)
		end
	end

	--[[ TODO: Possible use of GetQuestWorldMapAreaID? ]]--
	local qcMapID = 0
	local qcMapLevel = 0
	do
		SetMapToCurrentZone()
		if (GetCurrentMapAreaID() > 0) then
			qcMapID = GetCurrentMapAreaID()
		end
		if (GetCurrentMapDungeonLevel() > 0) then
			qcMapLevel = GetCurrentMapDungeonLevel()
		end
	end

	local qcMapIgnore = 0

	local qcFaction = 0
	do
		local qcEFaction, _S = UnitFactionGroup("player")
		qcFaction = qcFactionBits[string.upper(qcEFaction)]
	end

	local qcRace = 0
	do
		local _S, qcERace = UnitRace("player")
		qcRace = qcRaceBits[string.upper(qcERace)]
	end

	local qcClass = 0
	do
		local _S, qcEClass = UnitClass("player")
		qcClass = qcClassBits[string.upper(qcEClass)]
	end

	local qcQuestNotes = nil

	if (qcCollected[qcQuestID] == nil) then
		qcCollected[qcQuestID] = {}
		qcCollected[qcQuestID][1] = {}
		qcCollected[qcQuestID][2] = {}
		qcCollected[qcQuestID][3] = {}
		qcCollected[qcQuestID][4] = {}
		qcCollected[qcQuestID][5] = {}
		qcCollected[qcQuestID][6] = {}
		qcCollected[qcQuestID][1][1] = qcQuestID
		qcCollected[qcQuestID][1][2] = qcQuestName
		qcCollected[qcQuestID][1][3] = qcQuestLevel
		qcCollected[qcQuestID][1][4] = qcCategory
		qcCollected[qcQuestID][1][5] = qcQuestType
		qcCollected[qcQuestID][2][1] = qcZoneID
		--qcCollected[qcQuestID][3][1] = qcNPCID --[[ ON DETAIL ]]--
		--qcCollected[qcQuestID][3][2] = qcNPCName --[[ ON DETAIL ]]--
		qcCollected[qcQuestID][3][3] = qcNPCZone
		qcCollected[qcQuestID][3][4] = qcNPCX
		qcCollected[qcQuestID][3][5] = qcNPCY
		qcCollected[qcQuestID][3][6] = qcMapID
		qcCollected[qcQuestID][3][7] = qcMapLevel
		qcCollected[qcQuestID][3][8] = qcMapIgnore
		qcCollected[qcQuestID][4][1] = qcFaction
		qcCollected[qcQuestID][4][2] = qcRace
		qcCollected[qcQuestID][4][3] = qcClass
		qcCollected[qcQuestID][5][1] = qcQuestNotes
		do
			SelectQuestLogEntry(qcQuestLogIndex)
			local qcReputationCount = GetNumQuestLogRewardFactions()
			local qcFactionName, qcReputationAmount, qcFactionID, qcIsHeader, qcHasRep, _S
			local qcIndex = 0
			for qcReputationIndex = 1, qcReputationCount do
				qcFactionID, qcReputationAmount = GetQuestLogRewardFactionInfo(qcReputationIndex)
				qcFactionName, _S, _S, _S, _S, _S, _S, _S, qcIsHeader, _S, qcHasRep = GetFactionInfoByID(qcFactionID)
				if (qcFactionName and (not qcIsHeader or qcHasRep)) then
					qcIndex = (qcIndex + 1)
					qcReputationAmount = floor(qcReputationAmount / 100)
					qcCollected[qcQuestID][6][qcFactionID] = qcReputationAmount
				end
			end
		end
	else
		if (qcCollected[qcQuestID][1] == nil) then qcCollected[qcQuestID][1] = {} end
		if (qcCollected[qcQuestID][2] == nil) then qcCollected[qcQuestID][2] = {} end
		if (qcCollected[qcQuestID][3] == nil) then qcCollected[qcQuestID][3] = {} end
		if (qcCollected[qcQuestID][4] == nil) then qcCollected[qcQuestID][4] = {} end
		if (qcCollected[qcQuestID][5] == nil) then qcCollected[qcQuestID][5] = {} end
		if (qcCollected[qcQuestID][6] == nil) then qcCollected[qcQuestID][6] = {} end
		qcCollected[qcQuestID][1][1] = qcQuestID
		qcCollected[qcQuestID][1][2] = qcQuestName
		qcCollected[qcQuestID][1][3] = qcQuestLevel
		qcCollected[qcQuestID][1][4] = qcCategory
		qcCollected[qcQuestID][1][5] = qcQuestType
		qcCollected[qcQuestID][2][1] = qcZoneID
		--qcCollected[qcQuestID][3][1] = qcNPCID --[[ ON DETAIL ]]--
		--qcCollected[qcQuestID][3][2] = qcNPCName --[[ ON DETAIL ]]--
		qcCollected[qcQuestID][3][3] = qcNPCZone
		qcCollected[qcQuestID][3][4] = qcNPCX
		qcCollected[qcQuestID][3][5] = qcNPCY
		qcCollected[qcQuestID][3][6] = qcMapID
		qcCollected[qcQuestID][3][7] = qcMapLevel
		qcCollected[qcQuestID][3][8] = qcMapIgnore
		if (bit.band(qcCollected[qcQuestID][4][1], qcFaction) == 0) then
			qcCollected[qcQuestID][4][1] = tonumber(qcCollected[qcQuestID][4][1] + qcFaction)
		end
		if (bit.band(qcCollected[qcQuestID][4][2], qcRace) == 0) then
			qcCollected[qcQuestID][4][2] = tonumber(qcCollected[qcQuestID][4][2] + qcRace)
		end
		if (bit.band(qcCollected[qcQuestID][4][3], qcClass) == 0) then
			qcCollected[qcQuestID][4][3] = tonumber(qcCollected[qcQuestID][4][3] + qcClass)
		end
		qcCollected[qcQuestID][5][1] = qcQuestNotes
		do
			SelectQuestLogEntry(qcQuestLogIndex)
			local qcReputationCount = GetNumQuestLogRewardFactions()
			local qcFactionName, qcReputationAmount, qcFactionID, qcIsHeader, qcHasRep, _S
			local qcIndex = 0
			for qcReputationIndex = 1, qcReputationCount do
				qcFactionID, qcReputationAmount = GetQuestLogRewardFactionInfo(qcReputationIndex)
				qcFactionName, _S, _S, _S, _S, _S, _S, _S, qcIsHeader, _S, qcHasRep = GetFactionInfoByID(qcFactionID)
				if (qcFactionName and (not qcIsHeader or qcHasRep)) then
					qcIndex = (qcIndex + 1)
					qcReputationAmount = floor(qcReputationAmount / 100)
					qcCollected[qcQuestID][6][qcFactionID] = qcReputationAmount
				end
			end
		end
	end

end
end

qcSVData = {
}